"""
WebDebug constants and helper methods


WebDebug is a web development proxy server that captures and displays HTTP
requests and responses.
Copyright (C) 1999  Paul Clip.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""

### IMPORTS

import string
import whrandom

### CONSTANTS

# The prefix by which URL commands to WebDebug are recognized
WD_PREFIX = "/WEBDEBUG"
WD_PREFIX_LEN = len(WD_PREFIX)

TRUE = 1
YES = 1
FALSE = 0
NO = 0

# The current version WebDebug number
VERSION_NUMBER = 1.20

REQUEST = "request"
RESPONSE = "response"

# MessageList sort order constants
CURRENT_SORT_ORDER = "CURRENT"
REQUEST_SORT_ORDER = "REQUEST"
RESPONSE_SORT_ORDER = "RESPONSE"
SOCKET_SORT_ORDER = "SOCKET"

# The separator for the toSpreadsheet() calls
SEPARATOR = "\t"

# The separator for the addRand() and stripRand() calls
# This separator should not be present in the strings it gets added to
RAND_SEPARATOR = "_#_"


### Functions

def startsWith(str, snippet):
	""" returns TRUE when str starts with snippet """
	if string.find(str, snippet) == 0:
		return TRUE
	else:
		return FALSE

def addRand(s):
	""" adds a random number at the end of a string """
	return s + RAND_SEPARATOR + str(whrandom.random())

def stripRand(str):
	""" removes the random number added by addRand() """
	pos = string.rfind(str, RAND_SEPARATOR)
	if pos != -1:
		return str[:pos]
	else:
		return str
