"""
WebDebug Replay

Command:
	$> wdreplay.py <file> [-bv] [-po]
Where:
	<file> is a previously-saved WebDebug message list
	-bv tells wdreplay to interact with BroadVision (handle sessions, etc.)
	-po tells wdreplay to only request pages (not images, etc.)

WebDebug is a web development proxy server that captures and displays HTTP
requests and responses.
Copyright (C) 1999  Paul Clip.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""

### IMPORTS

from WDconst import *
import WDconfig
import WDserver
import WDhandler
import MessageList
import HTTPrequest
import HTTPresponse

import sys
import string
import cPickle
from socket import *
import time

### CONSTANTS

# The message list to load
MESSAGELIST = "" 

# Handle BroadVision characteristics
HANDLE_BV = NO

# Only download pages?
PAGES_ONLY = NO

# The BroadVision session id and engine id names
BVSESSIONID = "BV_SessionID="
BVENGINEID = "BV_EngineID="

USAGE = """
Command:
	$> wdreplay.py -f <file> [-bv]
Where:
	<file> is a previously-saved WebDebug message list
	-bv tells wdreplay to interact with BroadVision (handle sessions, etc.)
	-po tells wdreplay to only request pages (not images, etc.)
"""

### HELPER ROUTINES

def replaceString(str, b, e, rep):
	""" Returns str[b:e] with rep """
	before = str[:b]
	after = str[e:]
	return before + rep + after

def findBVEnd(str, start):
	""" Returns the end of the BV session or engine IDs """
	end = len(str)
	c = ord('1')
	while ((c >= ord('0') and c <= ord('z')) or c == ord('.')) and start < end:
		c = ord(str[start])
		start = start + 1
	return start - 1

def sessionizeRequest(req, session):
	""" Updates the request with the current session info """
	path = req.getPath()
	pos = string.find(path, BVSESSIONID)
	if pos != -1:
		end = string.find(path, '&', pos)
		if end == -1:
			end = len(path)
		path = replaceString(path, pos+len(BVSESSIONID), end, session[BVSESSIONID])
	pos = string.find(path, BVENGINEID)
	if pos != -1:
		end = string.find(path, '&', pos)
		if end == -1:
			end = len(path)
		path = replaceString(path, pos+len(BVENGINEID), end, session[BVENGINEID])
	req.path = path
	print "New path: %s\n" % req.getPath()
	return req

def getResponse(httpRequest, session):
	""" Connects to a remote host, submits the request and creates the response """
	print "getting %s" % httpRequest.getPath()
	s = socket(AF_INET, SOCK_STREAM)
	s.connect(httpRequest.getHost(), httpRequest.getPort())
	outsock = s.makefile("wb")
	insock = s.makefile("rb")
	httpRequest = sessionizeRequest(httpRequest, session)
	httpRequest.sendRequest(outsock)
	httpResponse = HTTPresponse.HTTPresponse(insock, s.getpeername())
	# print httpResponse.getRawContents()
	outsock.close()
	insock.close()
	s.close()
	return httpResponse

def logStats(request, response):
	""" Logs statistics associated with each page """
	pass

def populateSession(response):
	""" Returns BroadVision session & engine IDs """
	map = {}
	if response.getContentType() != 'text/html':
		return map
	page = response.getRawContents()
	pos = string.find(page, BVSESSIONID)
	if pos == -1:
		return map
	map[BVSESSIONID] = page[pos+len(BVSESSIONID) : string.find(page,'&', pos)]
	pos = string.find(page, BVENGINEID)
	if pos == -1:
		return {}
	map[BVENGINEID] = page[pos+len(BVENGINEID) : string.find(page,'"', pos)]
	print "Session is %s" % map
	return map

### MAIN SCRIPT

argerrors = NO

# Parse command line arguments
try:
	index = 1
	while index < len(sys.argv):
		if sys.argv[index] == '-f':
			MESSAGELIST = sys.argv[index+1]
			index = index + 2
		elif sys.argv[index] == '-bv':
			HANDLE_BV = YES
			print "Handling BroadVision"
			index = index + 1
		elif sys.argv[index] == '-po':
			PAGES_ONLY = YES
			print "Only downloading pages"
			index = index + 1
		else:
			print "Unknown parameter %s" % sys.argv[index]
			argerrors = YES
			index = index + 1
except:
	print "Error parsing arguments\n%s" % USAGE
	sys.exit(-1)
else:
	if argerrors:
		print "Error(s) in argument(s)\n%s" % USAGE
		sys.exit(-1)
		
# load MessageList
try:
	f = open(MESSAGELIST, 'r')
	MessageList.myList = cPickle.load(f)
	f.close()
	print("Message List loaded")
except:
	print("Could not load file: %s" % MESSAGELIST)
	sys.exit(-1)

print len(MessageList.myList.messagelist)

# Create empty session
session = {}

# Start the clock
STARTTIME = time.time()

# Re-run requests in message list
for m in MessageList.myList.messagelist:
	if PAGES_ONLY and m[RESPONSE].getContentType() != 'text/html':
		continue
	request = m[REQUEST]
	response = getResponse(request, session)
	logStats(request, response)
	if HANDLE_BV and session == {}:
		session = populateSession(response)
	print response.getContentType()





